@@ -1,6 +1,8 @@
 /* cache.h - Definitions for the dns cache
    Copyright (C) 2000 Thomas Moestl
 
+   With modifications by Paul Rombouts, 2003.
+
 This file is part of the pdnsd package.
 
 pdnsd is free software; you can redistribute it and/or modify
@@ -83,7 +85,7 @@ typedef struct {
 } dns_file_t;
 
 /*
- * This has two modes: Normally, we have rrest, cent and tp filled in;
+ * This has two modes: Normally, we have rrset, cent and tp filled in;
  * for negatively cached cents, we have rrset set to NULL and tp set to -1
  */
 typedef struct rr_lent_s {
@@ -122,37 +124,69 @@ typedef struct rr_lent_s {
  */
 #define CACHE_LAT 120
 
-void init_cache(void);
-void init_cache_lock(void);
+extern volatile int use_cache_lock;
+
+
+#ifdef ALLOC_DEBUG
+#define DBGPARAM ,int dbg
+#define DBGARG ,dbg
+#define DBG0 ,0
+#define DBG1 ,1
+#else
+#define DBGPARAM
+#define DBGARG
+#define DBG0
+#define DBG1
+#endif
+
+
+/* Initialize the cache. Call only once. */
+#define init_cache mk_dns_hash
+
+/* Initialize the cache lock. Call only once. */
+inline static void init_cache_lock()
+{
+	use_cache_lock=1;
+}
+
 void destroy_cache(void);
 void read_disk_cache(void);
 void write_disk_cache(void);
 
-void report_cache_stat(int f);
+int report_cache_stat(int f);
 
 /*
  *  add_cache expects the dns_cent_t to be filled.
  */
-void add_cache(dns_cent_t ent);
+void add_cache(dns_cent_t *cent);
 void del_cache(unsigned char *name);
 void invalidate_record(unsigned char *name);
 int have_cached(unsigned char *name);
 dns_cent_t *lookup_cache(unsigned char *name);
 int add_cache_rr_add(unsigned char *name, time_t ttl, time_t ts, short flags, int dlen, void *data, int tp, unsigned long serial);
 
-int mk_flag_val(servparm_t *server);
-int init_cent(dns_cent_t *cent, unsigned char *qname, short flags, time_t ts, time_t ttl, int dbg);
-int add_cent_rrset(dns_cent_t *cent,  int tp, time_t ttl, time_t ts, int flags, unsigned long serial, int dbg);
-int add_cent_rr(dns_cent_t *cent, time_t ttl, time_t ts, short flags,int dlen, void *data, int tp, int dbg);
-void free_cent(dns_cent_t cent, int dbg);
+inline static int mk_flag_val(servparm_t *server)
+{
+	int fl=0;
+	if (!server->purge_cache)
+		fl|=CF_NOPURGE;
+	if (server->nocache)
+		fl|=CF_NOCACHE;
+	return fl;
+}
+
+int init_cent(dns_cent_t *cent, unsigned char *qname, short flags, time_t ts, time_t ttl  DBGPARAM);
+int add_cent_rrset(dns_cent_t *cent,  int tp, time_t ttl, time_t ts, int flags, unsigned long serial  DBGPARAM);
+int add_cent_rr(dns_cent_t *cent, time_t ttl, time_t ts, short flags,int dlen, void *data, int tp  DBGPARAM);
+void free_cent(dns_cent_t *cent  DBGPARAM);
+void del_cent(dns_cent_t *cent);
 
 /* Because this is empty by now, it is defined as an empty macro to save overhead.*/
 /*void free_rr(rr_bucket_t cent);*/
-#define free_rr(x, dbg)
+#define free_rr(x)
 
-dns_cent_t *copy_cent(dns_cent_t *cent, int dbg);
-rr_bucket_t *copy_rr(rr_bucket_t *rr, int dbg);
-rr_bucket_t *create_rr(int dlen, void *data, int dbg);
+dns_cent_t *copy_cent(dns_cent_t *cent  DBGPARAM);
+rr_bucket_t *create_rr(int dlen, void *data  DBGPARAM);
 
 unsigned long get_serial(void);
 
