@@ -1,6 +1,8 @@
 /* dns.c - Declarations for dns handling and generic dns functions
    Copyright (C) 2000, 2001 Thomas Moestl
 
+   With modifications by Paul Rombouts, 2002, 2003, 2004.
+
 This file is part of the pdnsd package.
 
 pdnsd is free software; you can redistribute it and/or modify
@@ -34,7 +36,7 @@ static char rcsid[]="$Id: dns.c,v 1.31 2002/01/03 17:47:20 tmm Exp $";
 /* Decompress a name record, taking the whole message as msg, returning its results in tgt (max. 255 chars),
  * taking sz as the remaining msg size (it is returned decremented by the name length, ready for further use) and
  * a source pointer (it is returned pointing to the location after the name). msgsize is the size of the whole message,
- * len is the total name lentgh.
+ * len is the total name length.
  * msg and msgsz are needed for decompression (see rfc1035). The returned data is decompressed, but still in the
  * rr name form (length byte - string of that length, terminated by a 0 lenght byte).
  * If uscore is NULL, an underscore will be treated as illegal (except when UNDERSCORE is defined). Otherwise,
@@ -123,52 +125,49 @@ int decompress_name(unsigned char *msg, unsigned char *tgt, unsigned char **src,
 		}
 	}
 	*src+=osz-*sz;
-	*len=tpos;
+	if(len) *len=tpos;
 	return RC_OK;
 }
 
-/* Compare the names back-to-forth and return the longest match. The comparison is done at 
- * name granularity. The return value is the length of the match in name elements.
- * The unmatched part of ms is returned in rest (may be empty). o is set to the offset in the
- * domain name md (in length byte-string notation) of the match.
- * rest must point to a buffer of at least 256 bytes.
+/* Compare the names (in length byte-string notation) back-to-forth and return the longest match.
+   The comparison is done at name granularity.
+   The return value is the length of the match in name elements.
+   *os (*od) is set to the offset in the domain name ms (md) of the match.
  */
-int domain_match(int *o, unsigned char *ms, unsigned char *md, unsigned char *rest)
+int domain_match(const unsigned char *ms, const unsigned char *md, int *os, int *od)
 {
-	unsigned char sbuf[257],dbuf[257];
-	int offs, slen, dlen, cnt, nc;
+	int i,j,k,n,ns=0,nd=0,offs,offd;
+	unsigned char lb,ls[128],ld[128];
 
-	sbuf[0]='.';          /* Prefix the names with '.' : This is done for the special case that */
-	dbuf[0]='.';          /* the domains match exactly, or one is a complete subdomain of another */
-	rhn2str(ms,sbuf+1); /* Change to dotted notation since processing starts from behind, */
-	rhn2str(md,dbuf+1); /* and so it's much easier that way. */
-	/* If this is the root domain, we have two dots. bad. so this special case test: */
-	if (strcmp((char *)&sbuf[1],".")==0) {
-		*o=0;
-		rest[0]='\0';
-		return 0;
+	/* first collect all length bytes */
+	i=0;
+	while((lb=ms[i])) {
+		PDNSD_ASSERT(ns<128, "domain_match: too many name segments");
+		ls[ns++]=lb;
+		i += lb+1;
 	}
-	if (strlen((char *)sbuf)<2 || strlen((char *)dbuf)<2)
-		return 0;
-	slen=strlen((char *)sbuf)-2;
-	dlen=strlen((char *)dbuf)-2;
-	nc=cnt=0;
-	offs=-1;
-	while (cnt<=slen && cnt<=dlen) {
-		if (tolower(sbuf[slen-cnt])!=tolower(dbuf[dlen-cnt]))
-			break;
-		if (sbuf[slen-cnt]=='.') {
-			/* one complete name part matched. Set the offset */
-			nc++;
-			offs=cnt;
-		}
-		cnt++;
+
+	j=0;
+	while((lb=md[j])) {
+		PDNSD_ASSERT(nd<128, "domain_match: too many name segments");
+		ld[nd++]=lb;
+		j += lb+1;
 	}
-	*o=dlen-offs;
-	memset(rest,'\0',256);
-	if (slen-offs>0) 
-		memcpy(rest,ms,slen-offs);
-	return nc;
+
+	n=ns;  if(n>nd) n=nd;
+
+	for(k=1; offs=i,offd=j,k<=n; ++k) {
+		lb=ls[ns-k];
+		if(lb!=ld[nd-k]) goto mismatch;
+		for(;lb;--lb)
+			if(tolower(ms[--i]) != tolower(md[--j])) goto mismatch;
+		--i; --j;
+	}
+ mismatch:
+
+	if(os) *os=offs;
+	if(od) *od=offd;
+	return k-1;
 }
 
 /* compress the domain name in in and put the result (of maximum length of strlen(in)) and
@@ -179,144 +178,131 @@ int domain_match(int *o, unsigned char *ms, unsigned char *md, unsigned char *re
  * It is guaranteed (and insured by assertions) that the output is smaller or equal in
  * size to the input.
  */
-int compress_name(unsigned char *in, unsigned char *out, int offs, darray *cb)
+int compress_name(unsigned char *in, unsigned char *out, int offs, compel_array *cb)
 {
 	int i;
 	int add=1;
-	int coffs=-1;
-	int rv,rl,to;
-	int longest=0;
-	long ilen;
-	unsigned char rest[256];
-	unsigned char brest[256];
-	rl=0;
-	ilen = rhnlen(in);
+	int longest=0,lrem=0,coffs=0;
+	int rl=0;
+	unsigned ilen = rhnlen(in);
+
+	PDNSD_ASSERT(ilen<=256, "compress_name: name too long");
+
 	/* part 1: compression */
-	if (*cb) {
-		for (i=0;i<da_nel(*cb);i++) {
-			if ((rv=domain_match(&to, in, DA_INDEX(*cb,i,compel_t)->s,rest))>longest) {
-				/*
-				 * This has some not obvious implications that should be noted: If a 
-				 * domain name as saved in the list has been compressed, we only can
-				 * index the non-compressed part. We rely here that the first occurence
-				 * can't be compressed. So we take the first occurence of a given length.
-				 * This works perfectly, but watch it if you change something.
-				 */
-				rhncpy(brest,rest);
-				longest=rv;
-				coffs=DA_INDEX(*cb,i, compel_t)->index+to;
-			} 
+	for (i=0;i<DA_NEL(*cb);i++) {
+		int rv,rem,to;
+		if ((rv=domain_match(in, DA_INDEX(*cb,i).s, &rem,&to))>longest) {
+			/*
+			 * This has some not obvious implications that should be noted: If a 
+			 * domain name as saved in the list has been compressed, we only can
+			 * index the non-compressed part. We rely here that the first occurence
+			 * can't be compressed. So we take the first occurence of a given length.
+			 * This works perfectly, but watch it if you change something.
+			 */
+			longest=rv;
+			lrem=rem;
+			coffs=DA_INDEX(*cb,i).index+to;
 		}
-		if (coffs>-1) {
-			PDNSD_ASSERT(rhnlen(brest) + 1 <= ilen, "compress_name: length increased");
-			rl=rhncpy(out, brest)-1; /* omit the length byte, because it needs to be frobbed */
-			PDNSD_ASSERT(rl <= 254, "compress_name: name too long");
-			out[rl]=192|((coffs&0x3f00)>>8);
-			out[rl+1]=coffs&0xff;
-			rl+=2;
-			add=strlen((char *)brest)!=0;
-		} else
-			rl=rhncpy(out,in);
-	} else
-		rl=rhncpy(out,in);
+	}
+	if (longest>0) {
+		PDNSD_ASSERT(lrem+2 <= ilen, "compress_name: length increased");
+		memcpy(out, in,lrem);
+		out[lrem]=0xc0|((coffs&0x3f00)>>8);
+		out[lrem+1]=coffs&0xff;
+		rl=lrem+2;
+		add= lrem!=0;
+	}
+	else {
+		memcpy(out,in,ilen);
+		rl=ilen;
+	}
 
 	/* part 2: addition to the cache structure */
 	if (add) {
-		if (!*cb) {
-			if (!(*cb=DA_CREATE(compel_t)))
-			    return 0;
-		}
-		if (!(*cb=da_grow(*cb, 1)))
+		if (!(*cb=DA_GROW1(*cb)))
 			return 0;
-		DA_LAST(*cb, compel_t)->index=offs;
-		rhncpy(DA_LAST(*cb, compel_t)->s,in);
+		DA_LAST(*cb).index=offs;
+		memcpy(DA_LAST(*cb).s,in,ilen);
 	}
 	return rl;
 }
 
 
-typedef	struct {
-	struct in_addr ipv4;
-#ifdef ENABLE_IPV6
-	struct in6_addr ipv6;
-#endif
-} pdnsd_ca;
 
 /*
  * Add records for a host as read from a hosts-style file.
  * Returns 1 on success, 0 in an out of memory condition, and -1 when there was a problem with
  * the record data.
  */
-static int add_host(unsigned char *pn, unsigned char *rns, unsigned char *b3, pdnsd_ca *a, int a_sz, time_t ttl, int flags, int tp, int reverse)
+static int add_host(unsigned char *pn, unsigned char *rns, unsigned char *b3, pdnsd_a *a, int a_sz, time_t ttl, int flags, int tp, int reverse)
 {
 	dns_cent_t ce;
-	unsigned char b2[256],rhn[256];
-#if defined(DNS_NEW_RRS) && defined(ENABLE_IPV6)
-	unsigned char b4[7];
-	int i;
-#endif
 
-	if (!init_cent(&ce, pn, flags, time(NULL), 0, 0))
+	if (!init_cent(&ce, pn, flags, time(NULL), 0  DBG0))
 		return 0;
 #ifdef ENABLE_IPV4
 	if (tp==T_A) {
-		if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,a_sz,&a->ipv4,tp,0)) {
- 			free_cent(ce,0);
-			return 0;
-		}
+		if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,a_sz,&a->ipv4,tp  DBG0))
+			goto free_cent_return0;
 	}
 #endif
 #if defined(DNS_NEW_RRS) && defined(ENABLE_IPV6)
 	if (tp==T_AAAA) {
-		if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,a_sz,&a->ipv6,tp,0)) {
- 			free_cent(ce,0);
-			return 0;
-		}
+		if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,a_sz,&a->ipv6,tp  DBG0))
+			goto free_cent_return0;
 	}
 #endif
-	if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,rhnlen(rns),rns,T_NS,0)) {
-		free_cent(ce,0);
-		return 0;
-	}
-	add_cache(ce);
-	free_cent(ce,0);
+	if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,rhnlen(rns),rns,T_NS  DBG0))
+		goto free_cent_return0;
+	add_cache(&ce);
+	free_cent(&ce  DBG0);
 	if (reverse) {
+		unsigned char b2[256],rhn[256];
 #ifdef ENABLE_IPV4
-		if (tp==T_A) 
+		if (tp==T_A) {
+			uint32_t ipa=ntohl(a->ipv4.s_addr);
 # if TARGET==TARGET_BSD
-			snprintf((char *)b2,256,"%li.%li.%li.%li.in-addr.arpa.",ntohl(a->ipv4.s_addr)&0xffl,(ntohl(a->ipv4.s_addr)>>8)&0xffl,
-				 (ntohl(a->ipv4.s_addr)>>16)&0xffl, (ntohl(a->ipv4.s_addr)>>24)&0xffl);
+			snprintf(b2,sizeof(b2),"%li.%li.%li.%li.in-addr.arpa.",ipa&0xffl,(ipa>>8)&0xffl,
+				 (ipa>>16)&0xffl, (ipa>>24)&0xffl);
 # else
-			snprintf((char *)b2,256,"%i.%i.%i.%i.in-addr.arpa.",ntohl(a->ipv4.s_addr)&0xff,(ntohl(a->ipv4.s_addr)>>8)&0xff,
-				 (ntohl(a->ipv4.s_addr)>>16)&0xff, (ntohl(a->ipv4.s_addr)>>24)&0xff);
+			snprintf(b2,sizeof(b2),"%i.%i.%i.%i.in-addr.arpa.",ipa&0xff,(ipa>>8)&0xff,
+				 (ipa>>16)&0xff, (ipa>>24)&0xff);
 # endif
+		}
+		else
 #endif
 #if defined(DNS_NEW_RRS) && defined(ENABLE_IPV6)
 		if (tp==T_AAAA) {
-			b2[0]='\0';
-			for (i=15;i>=0;i--) {
-				snprintf((char *)b4, sizeof(b4),"%x.%x.",((unsigned char *)&a->ipv6)[i]&&0xf,(((unsigned char *)&a->ipv6)[i]&&0xf0)>>4);
-				strncat((char *)b2,(char *)b4,sizeof(b2)-strlen((char *)b2)-1);
+			int i,offs=0;
+			for (i=15;i>=0;--i) {
+				unsigned char bt=((unsigned char *)&a->ipv6)[i];
+				int n=snprintf(b2+offs, sizeof(b2)-offs,"%x.%x.",bt&0xf,(bt>>4)&0xf);
+				if(n<0) return -1;
+				offs+=n;
+				if(offs>=sizeof(b2)) return -1;
 			}
-			strncat((char *)b2,"ip6.int.",sizeof(b2)-strlen((char *)b2)-1);
+			if(!strncp(b2+offs,"ip6.int.",sizeof(b2)-offs))
+				return -1;
 		}
+		else
 #endif
+			return -1;
 		if (!str2rhn(b2,rhn))
 			return -1;
-		if (!init_cent(&ce, b2, flags, time(NULL), 0, 0))
+		if (!init_cent(&ce, b2, flags, time(NULL), 0  DBG0))
 			return 0;
-		if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,rhnlen(b3),b3,T_PTR,0)) {
- 			free_cent(ce,0);
-			return 0;
-		}
-		if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,rhnlen(rns),rns,T_NS,0)) {
- 			free_cent(ce,0);
-			return 0;
-		}
-		add_cache(ce);
-		free_cent(ce,0);
+		if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,rhnlen(b3),b3,T_PTR  DBG0))
+			goto free_cent_return0;
+		if (!add_cent_rr(&ce,ttl,0,CF_LOCAL,rhnlen(rns),rns,T_NS  DBG0))
+			goto free_cent_return0;
+		add_cache(&ce);
+		free_cent(&ce  DBG0);
 	}
 	return 1;
+
+ free_cent_return0:
+	free_cent(&ce  DBG0);
+	return 0;
 }
 
 /*
@@ -324,113 +310,117 @@ static int add_host(unsigned char *pn, unsigned char *rns, unsigned char *b3, pd
  * Errors are largely ignored so that we can skip entries we do not understand
  * (but others possibly do).
  */
-int read_hosts(char *fn, unsigned char *rns, time_t ttl, int flags, int aliases, char *errbuf, int errsize)
+int read_hosts(char *fn, unsigned char *rns, time_t ttl, int flags, int aliases, char **errstr)
 {
+	int rv=0;
 	FILE *f;
-	unsigned char buf[1025];
-	unsigned char b2[257],b3[256];
-	unsigned char *p,*pn,*pi,lastc;
-	struct in_addr ina4;
-	int tp;
-	int sz;
-	pdnsd_ca a;
+	char *buf;
+	size_t buflen=256;
 
-	buf[1023]='\0';
 	if (!(f=fopen(fn,"r"))) {
-		snprintf(errbuf, errsize, "Failed to source %s: %s", fn, strerror(errno));
+		if(asprintf(errstr, "Failed to source %s: %s", fn, strerror(errno))<0) *errstr=NULL;
 		return 0;
 	}
-	while (!feof(f)) {
-		if (fgets((char *)buf,sizeof(buf),f)==NULL) {
-			if (feof(f))
-				break;
-			snprintf(errbuf, errsize, "Failed to source %s: %s", fn, strerror(errno));
-			fclose(f);
-			return 0;
-		}
-		buf[sizeof(buf)-1]='\0';
+	buf=malloc(buflen);
+	if(!buf) {
+		*errstr=NULL;
+		goto fclose_return;
+	}
+	while(getline(&buf,&buflen,f)>=0) {
+		unsigned int len;
+		unsigned char *p,*pn,*pi;
+		unsigned char b2[256],b3[256];
+		int tp,sz;
+		pdnsd_a a;
+
+		p=strchr(buf,'#');
+		if(p) *p=0;
 		p=buf;
-		while (*p != '\0') {
-			if (*p=='#') {
-				*p='\0';
-				break;
-			}
-			p++;
+		for(;;) {
+			if(!*p) goto nextline;
+			if(!isspace(*p)) break;
+			++p;
 		}
-		pi=buf;
-		while (*pi==' ' || *pi=='\t') pi++;
-		if (*pi=='\0')
-			continue;
-		pn=pi;
-		while (*pn=='.' || *pn==':' || isxdigit(*pn)) pn++;  /* this includes IPv6 (':') */
-		if (*pn=='\0')
-			continue;
-		*pn='\0';
-		pn++;
-		while (*pn==' ' || *pn=='\t') pn++;
-		if (*pn=='\0')
-			continue;
-		p=pn;
-		while (isdchar(*p) || *p=='.') p++;
-		lastc=*p;
-		*p='\0';
-		memset(b2,'\0',257);
-		strncpy((char *)b2,(char *)pn,255);
-		if (b2[strlen((char *)b2)-1]!='.' && strlen((char *)b2)<255) {
-			b2[strlen((char *)b2)]='.';
+		pi=p;
+		do {
+			if(!*++p) goto nextline;
+		} while(!isspace(*p));
+		*p=0;
+		do {
+			if(!*++p) goto nextline;
+		} while (isspace(*p));
+		pn=p;
+		do {
+			++p;
+		} while(*p && !isspace(*p));
+		len=p-pn;
+		if(len>255) continue;
+		memcpy(b2,pn,len);
+		if(b2[len-1]!='.') {
+			b2[len]='.';
+			if(++len>255) continue;
 		}
+		b2[len]=0;
 		if (!str2rhn(b2,b3))
 			continue;
-		if (inet_aton((char *)pi,&ina4)) {
-			a.ipv4=ina4;
+		if (inet_aton(pi,&a.ipv4)) {
 			tp=T_A;
 			sz=sizeof(struct in_addr);
 		} else {
 #if defined(DNS_NEW_RRS) && defined(ENABLE_IPV6) /* We don't read them otherwise, as the C library may not be able to to that.*/
-			if (inet_pton(AF_INET6,(char *)pi,&a.ipv6)==1) {
+			if (inet_pton(AF_INET6,pi,&a.ipv6)>0) {
 				tp=T_AAAA;
 				sz=sizeof(struct in6_addr);
 			} else
-				continue;
-#else
-			continue;
 #endif
-		}
-		switch (add_host(b2, rns, b3, &a, sz, ttl, flags, tp,1)) {
-		case 0:
-			strncpy(errbuf, "Out of memory", errsize);
-			errbuf[errsize-1]='\0';
-			return 0;
-		case -1:
 			continue;
 		}
-		while (aliases) {
-			if (lastc=='\0')
-				break;
-			pn=p+1;
-			while (*pn==' ' || *pn=='\t') pn++;
-			if (*pn=='\0')
-				break;
-			p=pn;
-			while (isdchar(*p) || *p=='.') p++;
-			lastc=*p;
-			*p='\0';
-			memset(b2,'\0',257);
-			strncpy((char *)b2,(char *)pn,255);
-			if (b2[strlen((char *)b2)-1]!='.' && strlen((char *)b2)<255) {
-				b2[strlen((char *)b2)]='.';
+		{
+			int res=add_host(b2, rns, b3, &a, sz, ttl, flags, tp,1);
+			if(res==0) {
+				*errstr= NULL;
+				goto cleanup_return;
 			}
-			if (!str2rhn(b2,b3))
-				break;
-			if (add_host(b2, rns, b3, &a, sz, ttl, flags, tp,0) == 0) {
-				strncpy(errbuf, "Out of memory", errsize);
-				errbuf[errsize-1]='\0';
-				return 0;
+			else if(res<0)
+				continue;
+		}
+		if(aliases) {
+			for(;;) {
+				for(;;) {
+					if(!*p) goto nextline;
+					if(!isspace(*p)) break;
+					++p;
+				}
+				pn=p;
+				do {
+					++p;
+				} while(*p && !isspace(*p));
+				len=p-pn;
+				if(len>255) break;
+				memcpy(b2,pn,len);
+				if(b2[len-1]!='.') {
+					b2[len]='.';
+					if(++len>255) break;
+				}
+				b2[len]=0;
+				if (!str2rhn(b2,b3))
+					break;
+				if (add_host(b2, rns, b3, &a, sz, ttl, flags, tp,0) == 0) {
+					*errstr= NULL;
+					goto cleanup_return;
+				}
 			}
 		}
+	nextline:;
 	}
+	if (feof(f))
+		rv=1;
+	else if(asprintf(errstr, "Failed to source %s: %s", fn, strerror(errno))<0) *errstr=NULL;
+ cleanup_return:
+	free(buf);
+ fclose_return:
 	fclose(f);
-	return 1;
+	return rv;
 }
 
 
