@@ -1,6 +1,8 @@
 /* dns.h - Declarations for dns handling and generic dns functions
    Copyright (C) 2000, 2001 Thomas Moestl
 
+   With modifications by Paul Rombouts, 2002, 2003.
+
 This file is part of the pdnsd package.
 
 pdnsd is free software; you can redistribute it and/or modify
@@ -18,7 +20,7 @@ along with pdsnd; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
-/* $Id: dns.h,v 1.14 2002/01/01 23:54:49 tmm Exp $ */
+/* $Id: dns.h,v 1.15 2003/04/06 23:02:46 tmm Exp $ */
 
 #ifndef DNS_H
 #define DNS_H
@@ -92,10 +94,6 @@ Boston, MA 02111-1307, USA.  */
 #define OP_IQUERY   1
 #define OP_STATUS   2
 
-
-/* rfc2181 details that the ttl is a 32-bit integer, where the most significant bit is always 0.
- * for convenience and Unix compatablility we use a uint32_t, which satisfies these conditions if 
- * positive (which is ensured in the code) */
 typedef struct {
 	/* the name is the first field. It has variable length, so it can't be put in the struct */
 	uint16_t type      __attribute__((packed));
@@ -166,11 +164,14 @@ typedef struct {
 	unsigned char s[256];
 } compel_t;
 
-int decompress_name(unsigned char *msg, unsigned char *tgt, unsigned char **src, long *sz, long msgsz, int *len, int *uscore);
-int domain_match(int *o, unsigned char *ms, unsigned char *md, unsigned char *rest);
-int compress_name(unsigned char *in, unsigned char *out, int offs, darray *cb);
+typedef DYNAMIC_ARRAY(compel_t) *compel_array;
 
-int read_hosts(char *fn, unsigned char *rns, time_t ttl, int flags, int aliases, char *errbuf, int errsize);
+
+int decompress_name(unsigned char *msg, unsigned char *tgt, unsigned char **src, long *sz, long msgsz, int *len, int *uscore);
+int domain_match(const unsigned char *ms, const unsigned char *md, int *os, int *od);
+int compress_name(unsigned char *in, unsigned char *out, int offs, compel_array *cb);
+
+int read_hosts(char *fn, unsigned char *rns, time_t ttl, int flags, int aliases, char **errstr);
 
 #if DEBUG>0 
 char *get_cname(int id);
