@@ -1,6 +1,8 @@
 /* error.c - Error handling
    Copyright (C) 2000, 2001 Thomas Moestl
 
+   With modifications by Paul Rombouts, 2003.
+
 This file is part of the pdnsd package.
 
 pdnsd is free software; you can redistribute it and/or modify
@@ -25,6 +27,7 @@ Boston, MA 02111-1307, USA.  */
 #include <syslog.h>
 #include <pthread.h>
 #include <signal.h>
+#include <string.h>
 #include "error.h"
 #include "helpers.h"
 #include "conff.h"
@@ -33,36 +36,36 @@ Boston, MA 02111-1307, USA.  */
 static char rcsid[]="$Id: error.c,v 1.7 2001/06/03 11:00:54 tmm Exp $";
 #endif
 
-pthread_mutex_t loglock;
-int use_lock=0;
+pthread_mutex_t loglock = PTHREAD_MUTEX_INITIALIZER;
+volatile int use_log_lock=0;
 
 /*
  * Initialize a mutex for io-locking in order not to produce gibberish on
  * multiple simultaneous errors.
  */
-void init_log(void)
+/* This is now defined as an inline function in error.h */
+/* void init_log_lock(void)
 {
-	pthread_mutex_init(&loglock,NULL);
-	use_lock=1;
-}
+	use_log_lock=1;
+} */
 
 /* We crashed? Ooops... */
 void crash_msg(char *msg)
 {
 	log_error("%s", msg);
 	log_error("pdnsd probably crashed due to a bug. Please consider sending a bug");
-	log_error("report to tmoestl@gmx.net");
+	log_error("report to p.a.rombouts@home.nl or tmoestl@gmx.net");
 }
 
 /* Log an error. If we are a daemon, use the syslog. s is a format string like
  * in printf, the optional following arguments are the arguments like in printf */
 void log_error(char *s,...)
 {
-	int ul;
+	int ul=0;
 	va_list va;
-	va_start(va,s);
-	if (use_lock)
+	if (use_log_lock)
 		ul=softlock_mutex(&loglock);
+	va_start(va,s);
 	if (daemon_p) {
 		openlog("pdnsd",LOG_PID,LOG_DAEMON);
 		vsyslog(LOG_ERR,s,va);
@@ -72,20 +75,20 @@ void log_error(char *s,...)
 		vfprintf(stderr,s,va);
 		fprintf(stderr,"\n");
 	}
-	if (use_lock && ul)
-		pthread_mutex_unlock(&loglock);
 	va_end(va);
+	if (ul)
+		pthread_mutex_unlock(&loglock);
 }
 
 /* Log a warning. If we are a daemon, use the syslog. s is a format string like
  * in printf, the optional following arguments are the arguments like in printf */
 void log_warn(char *s, ...)
 {
-	int ul;
+	int ul=0;
 	va_list va;
-	va_start(va,s);
-	if (use_lock)
+	if (use_log_lock)
 		ul=softlock_mutex(&loglock);
+	va_start(va,s);
 	if (daemon_p) {
 		openlog("pdnsd",LOG_PID,LOG_DAEMON);
 		vsyslog(LOG_ERR,s,va);
@@ -95,9 +98,9 @@ void log_warn(char *s, ...)
 		vfprintf(stderr,s,va);
 		fprintf(stderr,"\n");
 	}
-	if (use_lock && ul)
-		pthread_mutex_unlock(&loglock);
 	va_end(va);
+	if (ul)
+		pthread_mutex_unlock(&loglock);
 }
 
 /* Log an info if level is <= the current verbosity level.
@@ -105,14 +108,13 @@ void log_warn(char *s, ...)
  * in printf, the optional following arguments are the arguments like in printf */
 void log_info(int level, char *s, ...)
 {
-	va_list va;
-	va_start(va,s);
 	if (level<=verbosity) {
-		if (use_lock)
+		va_list va;
+		if (use_log_lock)
 			if (!softlock_mutex(&loglock)) {
-				va_end(va);
 				return;
 			}
+		va_start(va,s);
 		if (daemon_p) {
 			openlog("pdnsd",LOG_PID,LOG_DAEMON);
 			vsyslog(LOG_INFO,s,va);
@@ -122,8 +124,8 @@ void log_info(int level, char *s, ...)
 			vfprintf(stderr,s,va);
 			fprintf(stderr,"\n");
 		}
-		if (use_lock)
+		va_end(va);
+		if (use_log_lock)
 			pthread_mutex_unlock(&loglock);
 	}
-	va_end(va);
 }
