@@ -1,6 +1,8 @@
 /* error.h - Error handling
    Copyright (C) 2000, 2001 Thomas Moestl
 
+   With modifications by Paul Rombouts, 2003.
+
 This file is part of the pdnsd package.
 
 pdnsd is free software; you can redistribute it and/or modify
@@ -31,13 +33,20 @@ Boston, MA 02111-1307, USA.  */
 
 #include "thread.h"
 #include "helpers.h"
+#include "pdnsd_assert.h"
 
 /* --- from error.c */
-volatile extern int waiting;
+extern volatile int waiting;
+extern volatile int use_log_lock;
 /* --- */
 
 void crash_msg(char *msg);
-void init_log(void);
+
+inline static void init_log_lock(void)
+{
+	use_log_lock=1;
+}
+
 void log_error(char *s,...) printfunc(1, 2);
 void log_warn(char *s, ...) printfunc(1, 2);
 void log_info(int level, char *s, ...) printfunc(2, 3);
@@ -48,8 +57,8 @@ void log_info(int level, char *s, ...) printfunc(2, 3);
  * want to save the overhead when DEBUG is not defined. 
  * debug_p needs to be defined (by including conff.h), or you
  * will get strange errors.
- * Dont forget your semicolon after the macro call, or you
- * will get other strange errors ;-)
+ * A macro call expands to a complete statement, so a semicolon after
+ * the macro call is redundant.
  * The arguments are normal printfs, so you know how to use the args
  */
 #if DEBUG>0
@@ -66,7 +75,7 @@ extern FILE *dbg_file;
 # if DEBUG > 0
 /* XXX: The timestamp generation makes this a little heavy-weight */
 #  define DEBUG_MSG_(c,args...)								\
-	do {										\
+	{										\
 		if (debug_p) {								\
 			char DM_ts[32];							\
 			time_t DM_tt = time(NULL);					\
@@ -80,12 +89,15 @@ extern FILE *dbg_file;
 			fprintf(dbg_file,args);						\
 			fflush(dbg_file);						\
 		}									\
-	} while (0)
+	}
 #  define DEBUG_MSG(args...)	DEBUG_MSG_(0,args)
 #  define DEBUG_MSGC(args...)	DEBUG_MSG_(1,args)
+#  define DEBUG_SOCKA_MSG(args...) {char _debugsockabuf[ADDRSTR_MAXLEN]; DEBUG_MSG(args);}
+#  define SOCKA2STR(a)          socka2str(a,_debugsockabuf,ADDRSTR_MAXLEN)
 # else
 #  define DEBUG_MSG(args...)
 #  define DEBUG_MSGC(args...)
+#  define DEBUG_SOCKA_MSG(args...)
 # endif	/* DEBUG > 0 */
 #else
 /* ANSI style. */
@@ -96,7 +108,7 @@ extern FILE *dbg_file;
  *      might make things even more messy...
  */
 #  define DEBUG_MSG_(c,...)								\
-	do {										\
+	{										\
 		if (debug_p) {								\
 			char DM_ts[32];							\
 			time_t DM_tt = time(NULL);					\
@@ -110,23 +122,17 @@ extern FILE *dbg_file;
 			fprintf(dbg_file,__VA_ARGS__);					\
 			fflush(dbg_file);						\
 		}									\
-	} while (0)
+	}
 
 #  define DEBUG_MSG(...)	DEBUG_MSG_(0,__VA_ARGS__)
 #  define DEBUG_MSGC(...)	DEBUG_MSG_(1,__VA_ARGS__)
+#  define DEBUG_SOCKA_MSG(...)  {char _debugsockabuf[ADDRSTR_MAXLEN]; DEBUG_MSG(__VA_ARGS__);}
+#  define SOCKA2STR(a)          socka2str(a,_debugsockabuf,ADDRSTR_MAXLEN)
 # else
 #  define DEBUG_MSG(...)
 #  define DEBUG_MSGC(...)
+#  define DEBUG_SOCKA_MSG(...)
 # endif	/* DEBUG > 0 */
 #endif
 
-/*
- * Assert macro, used in some places. For now, it should be always defined, not
- * only in the DEBUG case, to be on the safe side security-wise.
- */
-#define PDNSD_ASSERT(cond, msg)						\
-	do { if (!(cond)) {						\
-		log_error("%s:%d: %s", __FILE__, __LINE__, msg);	\
-		pdnsd_exit();						\
- 	} } while (0)
 #endif
