@@ -1,6 +1,8 @@
 /* helpers.c - Various helper functions
    Copyright (C) 2000, 2001 Thomas Moestl
 
+   With modifications by Paul Rombouts, 2002, 2003.
+
 This file is part of the pdnsd package.
 
 pdnsd is free software; you can redistribute it and/or modify
@@ -59,14 +61,12 @@ void pdnsd_exit()
 int softlock_mutex(pthread_mutex_t *mutex)
 {
 	int tr=0;
-	while (1)  {
-		if (pthread_mutex_trylock(mutex)==0)
-			return 1;
-		if (tr++>SOFTLOCK_MAXTRIES)
+	while(pthread_mutex_trylock(mutex)) {
+		if (++tr>=SOFTLOCK_MAXTRIES)
 			return 0;
 		usleep_r(10000);
 	}
-	return 0;
+	return 1;
 }
 
 /*
@@ -97,7 +97,7 @@ int run_as(char *user)
 /*
  * returns whether c is allowed in IN domain names
  */
-int isdchar (unsigned char c)
+/* int isdchar (unsigned char c)
 {
 	if ((c>='a' && c<='z') || (c>='A' && c<='Z') || (c>='0' && c<='9') || c=='-'
 #ifdef UNDERSCORE
@@ -106,58 +106,87 @@ int isdchar (unsigned char c)
 	   )
 		return 1;
 	return 0;
-}
+} */
 
 /*
- * Convert a string given in dotted notation to the transport format (lenght byte prepended
+ * Convert a string given in dotted notation to the transport format (length byte prepended
  * domain name parts, ended by a null length sequence)
+ * The memory areas referenced by str and rhn may not overlap.
+ * The buffer rhn points to is assumed to be 256 bytes in size.
  */
 int str2rhn(unsigned char *str, unsigned char *rhn)
 {
-	int i=0;
-	int cnt=0;
-	unsigned char buf[64];
-	int lb;
-	int tcnt=0;
-	
-	do {
-		lb=0;
-		if (lb + cnt >= 255)
-			return 0;
-		while(isdchar(str[lb+cnt])) {
-			if (lb>62)
-				return 0;
-			buf[lb]=str[lb+cnt];
-			lb++;
-			if (lb + cnt >= 255)
-				return 0;
+	int n=0,i,j;
+
+	for(i=0;;) {
+		int jlim,lb;
+		jlim=i+63;
+		if(jlim>254) jlim=254; /* 254 because the termination 0 has to follow */
+		for(j=i; isdchar(str[j]); ++j) {
+			if(j>=jlim) return 0;
+			rhn[j+1]=str[j];
 		}
-		if (str[lb+cnt]=='\0') {
-			if (lb==0) {
-				if (i==0)
-					return 0;
-				rhn[tcnt]='\0';
-				return 1;
-			}
-			return 0;
-		} else if (str[lb+cnt]=='.') {
-			i++;
-			if (lb+tcnt+1>255) /* 255 because the termination 0 has to follow */
-				return 0;
-			rhn[tcnt]=(unsigned char)lb;
-			tcnt++;
-			memcpy(rhn+tcnt,buf,lb);
-			tcnt+=lb;
-			cnt+=lb+1;
-		} else
+		if(!str[j]) break;
+		if(str[j]!='.') return 0;
+		lb=j-i;
+		if (lb>0) {
+			rhn[i]=(unsigned char)lb;
+			++n;
+			i = j+1;
+		}
+		else
 			return 0;
 				
-	} while (1);
+	}
+
+	rhn[i]=0;
+	if (j>i || n==0)
+		return 0;
+	return 1;
+}
+
+/*
+  parsestr2rhn is essentially the same as str2rhn, except that it doesn't look beyond
+  the first len chars in the input string. It also tolerates strings
+  not ending in a dot and returns a message in case of an error.
+ */
+char *parsestr2rhn(unsigned char *str, int len, unsigned char *rhn)
+{
+	int n=0,i=0,j;
+
+	do {
+		int jlim,lb;
+		jlim=i+63;
+		if(jlim>254) jlim=254;
+		for(j=i; j<len && str[j] && str[j]!='.'; ++j) {
+			if(!isdchar(str[j]))
+				return "Illegal character in domain name";
+			if(j>=jlim)
+				return "Domain name element too long";
+			rhn[j+1]=str[j];
+		}
+
+		lb=j-i;
+		if (lb>0) {
+			rhn[i]=(unsigned char)lb;
+			++n;
+			i = j+1;
+		}
+		else if(j<len && str[j])
+			return "Empty name element in domain name";
+		else
+			break;
+	} while(j<len && str[j]);
+
+	rhn[i]=0;
+	if(n==0)
+		return "Empty or root domain name not allowed";
+	return NULL;
 }
 
 /*
  * Take a host name as used in the dns transfer protocol (a length byte, followed by the
- * first part of the name, ..., followed by a 0 lenght byte), and return a string (in str,
+ * first part of the name, ..., followed by a 0 length byte), and return a string (in str,
  * length is the same as rhn) in the usual dotted notation. Length checking is done 
  * elsewhere (in decompress_name), this takes names from the cache which are validated.
  * The buffer str points to is assumed to be 256 bytes in size.
@@ -165,38 +194,43 @@ int str2rhn(unsigned char *str, unsigned char *rhn)
 void rhn2str(unsigned char *rhn, unsigned char *str)
 {
 	unsigned char lb;
-	int i;
-	int cnt=1;
 
-	str[0]='\0';
 	lb=rhn[0];
 	if (!lb) {
-		strcpy((char *)str,".");
-		return;
+		strcpy(str,".");
 	}
- 	while (lb) {
-		for (i=0;i<lb;i++) {
-			PDNSD_ASSERT(cnt < 255,
-			    "rhn2str: string length overflow");
-			str[cnt-1]=rhn[cnt];
-			cnt++;
-		}
-		PDNSD_ASSERT(cnt <= 255,
-		    "rhn2str: string length overflow");
-		str[cnt-1]='.';
-		str[cnt]='\0';
-		lb=rhn[cnt];
-		cnt++;
-		if (cnt>255)
-			break;
+	else {
+		int i=0;
+		do {
+			PDNSD_ASSERT(i+lb < 255,
+				     "rhn2str: string length overflow");
+			for (;lb;--lb) {
+				str[i]=rhn[i+1];
+				i++;
+			}
+			str[i]='.';
+			i++;
+			lb=rhn[i];
+		} while(lb);
+		str[i]='\0';
 	}
 }
 
-/* Return the length of a rhn. This is for better abstraction and could be a macro */
-int rhnlen(unsigned char *rhn)
+/* Return the length of a rhn. The definition has in fact been moved to helpers.h as an inline function.
+   Note added by Paul Rombouts:
+   Compared to the definition used by Thomas Moestl (strlen(rhn)+1), the following definition of rhnlen
+   may yield a different result in certain error situations (when a domain name segment contains null byte).
+*/
+/* unsigned int rhnlen(unsigned char *rhn)
 {
-	return strlen((char *)rhn)+1;
+	unsigned int i=0;
+	unsigned char lb;
+
+	while((lb=rhn[i]))
+		i+=lb+1;
+	return i+1;
 }
+*/
 
 /*
  * Non-validating rhn copy (use with checked or generated data only).
@@ -205,13 +239,12 @@ int rhnlen(unsigned char *rhn)
  * The answer assembly code uses this; it is guaranteed to not clobber anything
  * after the name.
  */
-int rhncpy(unsigned char *dst, unsigned char *src)
+unsigned int rhncpy(unsigned char *dst, unsigned char *src)
 {
-	int len;
+	unsigned int len = rhnlen(src);
 
-	len = rhnlen(src);
 	PDNSD_ASSERT(len<=256,"rhncpy: src too long!");
-	memcpy((char *)dst,(char *)src,len>256?256:len);
+	memcpy(dst,src,len>256?256:len);
 	return len;
 }
 
@@ -245,7 +278,7 @@ int str2pdnsd_a(char *addr, pdnsd_a *a)
 	return 0;
 }
 
-int is_inaddr_any(pdnsd_a *a)
+/* int is_inaddr_any(pdnsd_a *a)
 {
 #ifdef ENABLE_IPV4
 	if (run_ipv4) {
@@ -258,52 +291,53 @@ int is_inaddr_any(pdnsd_a *a)
 	}
 #endif
 	return 0;
-}
+} */
 
 /*
  * This is used for user output only, so it does not matter when an error occurs
- * and we leave the string empty.
  */
-char *pdnsd_a2str(pdnsd_a *a, char *str, int maxlen)
+const char *pdnsd_a2str(pdnsd_a *a, char *buf, int maxlen)
 {
+	const char *res=NULL;  /* Initialized to inhibit compiler warning */
 #ifdef ENABLE_IPV4
 	if (run_ipv4) {
-		strncpy(str,inet_ntoa(a->ipv4),maxlen);
-		str[maxlen-1]='\0';
+		if (!(res=inet_ntop(AF_INET,&a->ipv4,buf,maxlen))) {
+			log_error("inet_ntop: %s", strerror(errno));
+		}
 	}
 #endif
 #ifdef ENABLE_IPV6
 	if (run_ipv6) {
-		if (inet_ntop(AF_INET6,&a->ipv6,str,maxlen)==NULL) {
+		if (!(res=inet_ntop(AF_INET6,&a->ipv6,buf,maxlen))) {
 			log_error("inet_ntop: %s", strerror(errno));
-			str[0]='\0';
 		}
 	}
 #endif
-	return str;
+	return res?res:"?.?.?.?";
 }
 
 
-#ifdef DEBUG
+#if DEBUG>0
 /* This is a function only needed by dns_query.c in debug mode. */
 
-char *socka2str(struct sockaddr *a, char *str, int maxlen)
+const char *socka2str(struct sockaddr *a, char *buf, int maxlen)
 {
+	const char *res=NULL;  /* Initialized to inhibit compiler warning */
 #ifdef ENABLE_IPV4
 	if (run_ipv4) {
-		strncpy(str,inet_ntoa(((struct sockaddr_in *)a)->sin_addr),maxlen);
-		str[maxlen-1]='\0';
+		if (!(res=inet_ntop(AF_INET,&((struct sockaddr_in *)a)->sin_addr,buf,maxlen))) {
+			log_error("inet_ntop: %s", strerror(errno));
+		}
 	}
 #endif
 #ifdef ENABLE_IPV6
 	if (run_ipv6) {
-		if (inet_ntop(AF_INET6,&((struct sockaddr_in6 *)a)->sin6_addr,str,maxlen)==NULL) {
+		if (!(res=inet_ntop(AF_INET6,&((struct sockaddr_in6 *)a)->sin6_addr,buf,maxlen))) {
 			log_error("inet_ntop: %s", strerror(errno));
-			str[0]='\0';
 		}
 	}
 #endif
-	return str;
+	return res?res:"?.?.?.?";
 }
 
 #endif
@@ -318,7 +352,7 @@ char *socka2str(struct sockaddr *a, char *str, int maxlen)
 #endif
 
 #ifdef RANDOM_DEVICE
-static FILE *rand_file;
+FILE *rand_file;
 #endif
 
 #ifdef R_RANDOM
@@ -337,7 +371,7 @@ int init_rng()
 {
 #ifdef RANDOM_DEVICE
 	if (!(rand_file=fopen(RANDOM_DEVICE,"r"))) {
-		log_error("Could not open %s.");
+		log_error("Could not open %s.",RANDOM_DEVICE);
 		return 0;
 	}
 #endif
@@ -347,13 +381,14 @@ int init_rng()
 	return 1;
 }
 
-void free_rng()
+/* The following function is now actually defined as a macro in helpers.h */
+/* void free_rng()
 {
 #ifdef RANDOM_DEVICE
 	if (rand_file)
 		fclose(rand_file);
 #endif
-}
+} */
 
 /* generate a (more or less) random number. */
 unsigned short get_rand16()
@@ -378,23 +413,42 @@ unsigned short get_rand16()
 #endif
 }
 
-void fsprintf(int fd, char *format, ...)
+/* the following function has been rewritten by Paul Rombouts */
+int fsprintf(int fd, const char *format, ...)
 {
-	char buf[1024];
-
+	int n;
 	va_list va;
-	va_start(va,format);
-	vsnprintf(buf,sizeof(buf),format,va);
-	write(fd,buf,strlen(buf));
 
-	va_end(va);
+	{
+		char buf[256];
+
+		va_start(va,format);
+		n=vsnprintf(buf,sizeof(buf),format,va);
+		va_end(va);
+
+		if(n<sizeof(buf)) {
+			if(n>0) n=write_all(fd,buf,n);
+			return n;
+		}
+	}
+	/* retry with a right sized buffer, needs glibc 2.1 or higher to work */
+	{
+		char buf[n+1];
+
+		va_start(va,format);
+		n=vsnprintf(buf,sizeof(buf),format,va);
+		va_end(va);
+
+		n=write_all(fd,buf,n);
+	}
+	return n;
 }
 
 /*
  * This is not like strcmp, but will return 1 on match or 0 if the
  * strings are different.
  */
-int stricomp(char *a, char *b)
+/* int stricomp(char *a, char *b)
 {
 	int i;
 	if (strlen(a) != strlen(b)) 
@@ -404,10 +458,10 @@ int stricomp(char *a, char *b)
 			return 0;
 	}
 	return 1;
-}
+} */
 
 /* Bah. I want strlcpy */
-int strncp(char *dst, char *src, int dstsz)
+/*int strncp(char *dst, char *src, int dstsz)
 {
 	char o;
 	
@@ -417,4 +471,91 @@ int strncp(char *dst, char *src, int dstsz)
 	if (strlen(dst) >= dstsz-1 && o!='\0')
 		return 0;
 	return 1;
+} */
+
+#ifndef HAVE_GETLINE
+/* Note by Paul Rombouts: I know that getline is a GNU extension and is not really portable,
+   but the alternative standard functions have some real problems.
+   The following substitute does not have exactly the same semantics as the GNU getline,
+   but it should be good enough, as long as the stream doesn't contain any null chars.
+   This version is actually based on fgets_realloc() that I found in the WWWOFFLE source.
+*/
+
+#define BUFSIZE 256
+int getline(char **lineptr, size_t *n, FILE *stream)
+{
+	char *line=*lineptr;
+	size_t sz=*n,i;
+
+	if(!line || sz<BUFSIZE) {
+		sz=BUFSIZE;
+		line = realloc(line,sz);
+		if(!line) return -1;
+		*lineptr=line;
+		*n=sz;
+	}
+
+	for (i=0;;) {
+		char *lni;
+
+		if(!(lni=fgets(line+i,sz-i,stream))) {
+			if(i && feof(stream))
+				break;
+			else
+				return -1;
+		}
+			
+		i += strlen(lni);
+
+		if(i<sz-1 || line[i-1]=='\n')
+			break;
+
+		sz += BUFSIZE;
+		line = realloc(line,sz);
+		if(!line) return -1;
+		*lineptr=line;
+		*n=sz;
+	}
+
+	return i;
 }
+#undef BUFSIZE
+#endif
+
+#ifndef HAVE_ASPRINTF
+int asprintf (char **lineptr, const char *format, ...)
+{
+	int sz=128,n;
+	char *line=malloc(sz);
+	va_list va;
+
+	if(!line) return -1;
+
+	va_start(va,format);
+	n=vsnprintf(line,sz,format,va);
+	va_end(va);
+
+	if(n>=sz) {
+		/* retry with a right sized buffer, needs glibc 2.1 or higher to work */
+		sz=n+1;
+		{
+			char *tmp=realloc(line,sz);
+			if(!tmp) {
+				free(line);
+				return -1;
+			}
+			line=tmp;
+		}
+
+		va_start(va,format);
+		n=vsnprintf(line,sz,format,va);
+		va_end(va);
+	}
+
+	if(n>=0)
+		*lineptr=line;
+	else
+		free(line);
+	return n;
+}
+#endif
