@@ -1,6 +1,8 @@
 /* netdev.c - Test network devices for existence and status
    Copyright (C) 2000, 2001 Thomas Moestl
 
+   With modifications by Paul Rombouts, 2002, 2003.
+
 This file is part of the pdnsd package.
 
 pdnsd is free software; you can redistribute it and/or modify
@@ -65,6 +67,7 @@ Boston, MA 02111-1307, USA.  */
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
+#include "helpers.h"
 #include "netdev.h"
 #include "error.h"
 
@@ -113,8 +116,7 @@ int statusif(char *name)
 		return 0;
 	}
 		
-	memset(&phone, 0, sizeof(phone));
-	strncpy(phone.name, name, sizeof(phone.name)-1);
+	strncp(phone.name, name, sizeof(phone.name));
 	if (ioctl(isdninfo, IIOCNETGPN, &phone)==0)
 		rc=1;
 	close(isdninfo);
@@ -129,31 +131,34 @@ int statusif(char *name)
  */
 int dev_up(char *ifname, char *devname)
 {
-	char buffer[256];
  	FILE *fd;
  	int pidi, pidd, rv;
 	
- 	if (snprintf(buffer, sizeof(buffer), "/var/run/%s.pid", ifname)>=sizeof(buffer))
-		return 0;
- 	if ((fd=fopen(buffer, "r")) == NULL )
- 		return 0;
+	{
+	  char path[sizeof("/var/run/.pid")+strlen(ifname)];
+	  stpcpy(stpcpy(stpcpy(path,"/var/run/"),ifname),".pid");
+	  if ((fd=fopen(path, "r")) == NULL )
+	    return 0;
 
- 	if (fscanf(fd, "%d", &pidi) != 1 ) {
-		fclose(fd) ;
- 		return 0;
- 	}
- 	fclose(fd);
- 
- 	if (snprintf(buffer, sizeof(buffer), "/var/lock/LCK..%s", devname)>=sizeof(buffer))
-		return 0;
- 	if ((fd=fopen(buffer, "r")) == NULL)
-		return 0;
+	  if (fscanf(fd, "%d", &pidi) != 1 ) {
+	    fclose(fd) ;
+	    return 0;
+	  }
+	  fclose(fd);
+	}
+
+	{
+	  char path[sizeof("/var/lock/LCK..")+strlen(devname)];
+	  stpcpy(stpcpy(path,"/var/lock/LCK.."),devname);
+	  if ((fd=fopen(path, "r")) == NULL)
+	    return 0;
 	
- 	if (fscanf(fd, "%d", &pidd) != 1) {
-		fclose(fd);
-		return 0;
- 	}
- 	fclose(fd);
+	  if (fscanf(fd, "%d", &pidd) != 1) {
+	    fclose(fd);
+	    return 0;
+	  }
+	  fclose(fd);
+	}
 	
  	if (pidi != pidd)
 		return 0;
@@ -181,7 +186,8 @@ int if_up(char *devname)
 	int sock;
 	struct ifreq ifr;
 # if TARGET==TARGET_LINUX
-	if (strlen(devname)>4 && strlen(devname)<=6 && strncmp(devname,"ippp",4)==0) {
+	unsigned int devnamelen=strlen(devname);
+	if (devnamelen>4 && devnamelen<=6 && strncmp(devname,"ippp",4)==0) {
 		/* This function didn't manage the interface uptest correctly. Thanks to
 		 * Joachim Dorner for pointing out. 
 		 * The new code (statusif()) was shamelessly stolen from isdnctrl.c of the 
@@ -202,8 +208,7 @@ int if_up(char *devname)
 		return 0;
 	if ((sock=socket(PF_INET,SOCK_DGRAM, pe->p_proto))==-1)
 		return 0;
-	strncpy(ifr.ifr_name,devname,IFNAMSIZ);
-	ifr.ifr_name[IFNAMSIZ-1]='\0';
+	strncp(ifr.ifr_name,devname,IFNAMSIZ);
 	if (ioctl(sock,SIOCGIFFLAGS,&ifr)==-1) {
 		close(sock);
 		return 0;
@@ -216,20 +221,11 @@ int if_up(char *devname)
 
 int is_local_addr(pdnsd_a *a)
 {
-	int i,res;
-#  ifdef ENABLE_IPV4
-	struct protoent *pe;
-	int sock;
-	struct ifreq ifr;
-#  endif
-#  ifdef ENABLE_IPV6
-	char   buf[50];
-	FILE   *f;
-	struct in6_addr b;
-#  endif
 #  ifdef ENABLE_IPV4
 	if (run_ipv4) {
-		res=0;
+		int res=0,i,sock;
+		struct protoent *pe;
+		struct ifreq ifr;
 		if (!(pe=getprotobyname("udp")))
 			return 0;
 		if ((sock=socket(PF_INET,SOCK_DGRAM, pe->p_proto))==-1)
@@ -255,36 +251,45 @@ int is_local_addr(pdnsd_a *a)
 #  endif
 #  ifdef ENABLE_IPV6
 	if (run_ipv6) {
+		int res=0;
+		char   buf[40];
+		FILE   *f;
+		struct in6_addr b;
 		/* the interface configuration and information retrieval is obiously currently done via 
 		 * rt-netlink sockets. I think it is relatively likely to change in an incompatible way the 
 		 * Linux kernel (there seem to be some major changes for 2.4).
 		 * Right now, I just analyze the /proc/net/if_inet6 entry. This may not be the fastest, but 
-		 * should work and is easily to adapt should the format change. */
+		 * should work and is easy to adapt should the format change. */
 		if (!(f=fopen("/proc/net/if_inet6","r")))
 			return 0;
-		/* The address is at the start of the line. We just read 4 characters and insert a ':' 8 
+		/* The address is at the start of the line. We just read 4 characters and insert a ':' 7 
 		 * times. Such, we can use inet_pton conveniently. More portable, that. */
-		while (!feof(f)) {
-			memset(buf,'\0',50);
+		for(;;) {
+			int i,ch;
 			for (i=0;i<8;i++) {
-				for (res=0;res<4;res++) {
-					if ((buf[i*5+res]=fgetc(f))==EOF) {
-						fclose(f);
-						return 0; /* we are at the end of the file and haven't found anything.*/
-					}
+				int j;
+				for (j=0;j<4;j++) {
+					if ((ch=fgetc(f))==EOF)
+						goto fclose_return; /* we are at the end of the file and haven't found anything.*/
+					if(ch=='\n') goto nextline;
+					buf[i*5+j]=ch;
 				}
-				if (i<7)
-					buf[i*5+4]=':';
+				buf[i*5+4]= (i<7 ? ':' : '\0');
 			}
 			if (inet_pton(AF_INET6,buf,&b) == 1) {
 				if (IN6_ARE_ADDR_EQUAL((&a->ipv6),(&b))) {
-					fclose(f);
-					return 1;
+					res=1;
+					goto fclose_return;
 				}
 			}
-			while ((res=fgetc(f))!='\n' && res!=EOF) ;
+			do {
+				if ((ch=fgetc(f))==EOF) goto fclose_return;
+			} while(ch!='\n');
+		nextline:;
 		}
+	fclose_return:
 		fclose(f);
+		return res;
 	}
 #  endif
 	return 0;
