@@ -1,6 +1,7 @@
 #include <config.h>
 #include <stdio.h>
 #include <pthread.h>
+#include <string.h>
 #include "../helpers.h"
 
 #if !defined(lint) && !defined(NO_RCSIDS)
@@ -12,7 +13,9 @@ int debug_p=0;
 int verbosity=VERBOSITY;
 pthread_t main_thread;
 
+#if defined(ENABLE_IPV4) && defined(ENABLE_IPV6)
 int run_ipv4=DEFAULT_IPV4;
+#endif
 int run_ipv6=DEFAULT_IPV6;
 
 int main(void) 
